unit Wizard;

// wizard.pas
//
//     Copyright(c) 2009-2011 PT2K <PT2K@d8.dion.ne.jp>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, ComCtrls, StdCtrls, ColorBox,
  ResStrs, B2MConv;

type

  { TB2MWizard }

  TB2MWizard = class(TForm)
    WizardNotebook: TNotebook;
      BitmapPathPage: TPage;
        BitmapPathLabel: TLabel;
        BitmapPathEdit: TEdit;
        BrowsBitmapPathButton: TButton;
        PreviewPanel: TPanel;
          PreviewImage: TImage;
          PreviewScrollBar: TScrollBar;
      UseTransparentColorPage: TPage;
        UseTransparentColorCheckBox: TCheckBox;
      TransparentColorPage: TPage;
        TransparentColorLabel: TLabel;
        TransparentColorListBox: TColorListBox;
      OffsetPage: TPage;
        OffsetLabel: TLabel;
        OffsetEdit: TEdit;
        OffsetUpDown: TUpDown;
      NoteTypePage: TPage;
        NoteTypeLabel: TLabel;
        NoteTypeComboBox: TComboBox;
        NoteLengthLabel: TLabel;
        NoteLengthEdit: TEdit;
        TempoLabel: TLabel;
        TempoEdit: TEdit;
        TempoUpDown: TUpDown;
      ProgressPage: TPage;
        ProgressBar: TProgressBar;
      MusPathPage: TPage;
        MusPathLabel: TLabel;
        MusPathEdit: TEdit;
        BrowsMusPathButton: TButton;
        MusMemo: TMemo;
    ButtonPanel: TPanel;
      PrevButton: TButton;
      NextButton: TButton;
      CancelButton: TButton;
    procedure BitmapPathEditChange(Sender: TObject);
    procedure BrowsBitmapPathButtonClick(Sender: TObject);
    procedure BrowsMusPathButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDropFiles(Sender: TObject; const FileNames: array of String);
    procedure MusPathEditChange(Sender: TObject);
    procedure PreviewImageResize(Sender: TObject);
    procedure PreviewPanelResize(Sender: TObject);
    procedure PreviewScrollBarScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
  private
    procedure AdjustPreviewScrollBar();
    procedure BitmapPathNextButtonClick(Sender: TObject);
    procedure BitmapPathPrevButtonClick(Sender: TObject);
    procedure Bmp2MusProgress(Progress: Integer);
    procedure GoBitmapPath();
    procedure GoMusPath();
    procedure GoNoteType();
    procedure GoOffset();
    procedure GoProgress();
    procedure GoTransparentColor();
    procedure GoUseTransparentColor();
    procedure MusPathNextButtonClick(Sender: TObject);
    procedure MusPathPrevButtonClick(Sender: TObject);
    procedure NoteTypeNextButtonClick(Sender: TObject);
    procedure NoteTypePrevButtonClick(Sender: TObject);
    procedure OffsetNextButtonClick(Sender: TObject);
    procedure OffsetPrevButtonClick(Sender: TObject);
    procedure PrepareNoteTypeComboBox();
    procedure SetResourceStrings();
    procedure TransparentColorNextButtonClick(Sender: TObject);
    procedure TransparentColorPrevButtonClick(Sender: TObject);
    procedure UseTransparentColorNextButtonClick(Sender: TObject);
    procedure UseTransparentColorPrevButtonClick(Sender: TObject);
  public
    procedure Execute();
  end;

var
  B2MWizard: TB2MWizard;

implementation

{ TB2MWizard }

procedure TB2MWizard.BitmapPathEditChange(Sender: TObject);
begin
  PreviewImage.Picture.Clear();
  NextButton.Enabled := False;
  if SetBitmapPath(BitmapPathEdit.Text) then
  begin
    PreviewImage.Picture.Assign(SourceBitmap);
    NextButton.Enabled := True;
  end;
end;

procedure TB2MWizard.BrowsBitmapPathButtonClick(Sender: TObject);
var
  BitmapPathOpenDialog: TOpenDialog;
begin
  BitmapPathOpenDialog := TOpenDialog.Create(Self);
  try
    with BitmapPathOpenDialog do
    begin
      Options := [ofAutoPreview, ofEnableSizing, ofFileMustExist, ofHideReadOnly,
                  ofPathMustExist, ofViewDetail];
      DefaultExt := 'bmp';
      Filter := resBitmapPathOpenDialogFilter;
      FilterIndex := 1;
      Title := resBitmapPathOpenDialogTitle;
      InitialDir := ExtractFilePath(BitmapPath);
      FileName := ExtractFileName(BitmapPath);
    end;
    if BitmapPathOpenDialog.Execute() then
    begin
      TransparentColorIndex := 0;
      BitmapPathEdit.Text := BitmapPathOpenDialog.FileName;
    end;
  finally
    BitmapPathOpenDialog.Free();
  end;
end;

procedure TB2MWizard.BrowsMusPathButtonClick(Sender: TObject);
var
  MusPathSaveDialog: TSaveDialog;
begin
  MusPathSaveDialog := TSaveDialog.Create(Self);
  try
    with MusPathSaveDialog do
    begin
      Options := [ofEnableSizing, ofHideReadOnly, ofOverwritePrompt,
                  ofPathMustExist, ofViewDetail];
      DefaultExt := 'mus';
      Filter := resMusPathSaveDialogFilter;
      FilterIndex := 1;
      Title := resMusPathSaveDialogTitle;
      InitialDir := ExtractFilePath(MusPath);
      FileName := ExtractFileName(MusPath);
    end;
    if MusPathSaveDialog.Execute() then
      MusPathEdit.Text := MusPathSaveDialog.FileName;
  finally
    MusPathSaveDialog.Free();
  end;
end;

procedure TB2MWizard.CancelButtonClick(Sender: TObject);
begin
  Close();
end;

procedure TB2MWizard.FormCreate(Sender: TObject);
begin
  PrepareNoteTypeComboBox();
  SetResourceStrings();
  OnB2MProgress := @Bmp2MusProgress;
  Execute();
end;

procedure TB2MWizard.FormDropFiles(Sender: TObject;
  const FileNames: array of String);
begin
  if 1 < Length(FileNames) then
    DoWarning(msgWTooMuchFiles, resWTooMuchFiles);
  TransparentColorIndex := 0;
  SetBitmapPath(FileNames[0]);
  GoBitmapPath();
end;

procedure TB2MWizard.MusPathEditChange(Sender: TObject);
begin
  MusPath := MusPathEdit.Text;
  NextButton.Enabled := '' <> MusPath;
end;

procedure TB2MWizard.PreviewImageResize(Sender: TObject);
begin
  AdjustPreviewScrollBar();
end;

procedure TB2MWizard.PreviewPanelResize(Sender: TObject);
begin
  AdjustPreviewScrollBar();
end;

procedure TB2MWizard.PreviewScrollBarScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  PreviewImage.Left := -ScrollPos;
end;

procedure TB2MWizard.AdjustPreviewScrollBar();
var
  w: Integer;
begin
  w := PreviewImage.Width - PreviewPanel.Width;
  if w < 0 then w := 0;
  PreviewScrollBar.Max := w;
end;

procedure TB2MWizard.BitmapPathNextButtonClick(Sender: TObject);
begin
  if ForceUseTransparentColor() then
    GoTransparentColor()
  else
    GoUseTransparentColor();
end;

procedure TB2MWizard.BitmapPathPrevButtonClick(Sender: TObject);
begin
  ;
end;

procedure TB2MWizard.Bmp2MusProgress(Progress: Integer);
begin
  ProgressBar.Position := Progress;
  Application.ProcessMessages();
end;

procedure TB2MWizard.GoBitmapPath();
begin
  WizardNotebook.ActivePage := 'BitmapPathPage';
  PrevButton.Enabled := False;
  NextButton.Enabled := False;
  NextButton.Caption := resNextButtonCaption;
  PrevButton.OnClick := @BitmapPathPrevButtonClick;
  NextButton.OnClick := @BitmapPathNextButtonClick;
  if FileExistsUTF8(BitmapPath) then
  begin
    BitmapPathEdit.Text := BitmapPath;
    BitmapPathEditChange(BitmapPathEdit);
  end;
end;

procedure TB2MWizard.GoMusPath();
var
  MusPathBak: TFileName;
begin
  WizardNotebook.ActivePage := 'MusPathPage';
  PrevButton.Enabled := True;
  NextButton.Enabled := False;
  NextButton.Caption := resFinishButtonCaption;
  PrevButton.OnClick := @MusPathPrevButtonClick;
  NextButton.OnClick := @MusPathNextButtonClick;
  MusPathBak := MusPath;
  MusPathEdit.Text := '';
  MusPathEdit.Text := MusPathBak;
  MusMemo.Lines.Assign(MusData);
end;

procedure TB2MWizard.GoNoteType();
begin
  WizardNotebook.ActivePage := 'NoteTypePage';
  PrevButton.Enabled := True;
  NextButton.Enabled := True;
  NextButton.Caption := resNextButtonCaption;
  PrevButton.OnClick := @NoteTypePrevButtonClick;
  NextButton.OnClick := @NoteTypeNextButtonClick;
  NoteTypeComboBox.ItemIndex := Ord(NoteType);
  NoteLengthEdit.Text := NoteLength;
  TempoUpDown.Position := Tempo;
end;

procedure TB2MWizard.GoOffset();
begin
  WizardNotebook.ActivePage := 'OffsetPage';
  PrevButton.Enabled := True;
  NextButton.Enabled := True;
  NextButton.Caption := resNextButtonCaption;
  PrevButton.OnClick := @OffsetPrevButtonClick;
  NextButton.OnClick := @OffsetNextButtonClick;
  OffsetUpDown.Max := MaxOffset;
  OffsetUpDown.Position := Offset;
end;

procedure TB2MWizard.GoProgress();
begin
  WizardNotebook.ActivePage := 'ProgressPage';
  PrevButton.Enabled := False;
  NextButton.Enabled := False;
  CancelButton.Enabled := False;
  NextButton.Caption := resNextButtonCaption;
  BitmapToMus();
  GoMusPath();
  CancelButton.Enabled := True;
end;

procedure TB2MWizard.GoTransparentColor();
var
  i: Integer;
  C: TColor;
begin
  WizardNotebook.ActivePage := 'TransparentColorPage';
  PrevButton.Enabled := True;
  NextButton.Enabled := True;
  NextButton.Caption := resNextButtonCaption;
  PrevButton.OnClick := @TransparentColorPrevButtonClick;
  NextButton.OnClick := @TransparentColorNextButtonClick;
  TransparentColorListBox.Clear();
  for i := 0 to ColorMapSize - 1 do
  begin
    C := ColorMap[i];
    TransparentColorListBox.Items.AddObject(
      Format('R:%.2x G:%.2x B:%.2x', [C and $FF, C and $FF00 shr 8, C and $FF0000 shr 16]),
      TObject(PtrInt(C))
    );
  end;
  TransparentColorListBox.ItemIndex := TransparentColorIndex;
end;

procedure TB2MWizard.GoUseTransparentColor();
begin
  WizardNotebook.ActivePage := 'UseTransparentColorPage';
  PrevButton.Enabled := True;
  NextButton.Enabled := True;
  NextButton.Caption := resNextButtonCaption;
  PrevButton.OnClick := @UseTransparentColorPrevButtonClick;
  NextButton.OnClick := @UseTransparentColorNextButtonClick;
  UseTransparentColorCheckBox.Checked := UseTransparentColor;
  UseTransparentColorCheckBox.Enabled := not ForceUseTransparentColor();
end;

procedure TB2MWizard.MusPathNextButtonClick(Sender: TObject);
begin
  SaveMus();
  Close();
end;

procedure TB2MWizard.MusPathPrevButtonClick(Sender: TObject);
begin
  GoNoteType();
end;

procedure TB2MWizard.NoteTypeNextButtonClick(Sender: TObject);
begin
  NoteType := TNoteType(NoteTypeComboBox.ItemIndex);
  NoteLength := NoteLengthEdit.Text;
  Tempo := TempoUpDown.Position;
  GoProgress();
end;

procedure TB2MWizard.NoteTypePrevButtonClick(Sender: TObject);
begin
  if CanSetOffset() then
    GoOffset()
  else if UseTransparentColor then
    GoTransparentColor()
  else if ForceUseTransparentColor() then
    GoBitmapPath()
  else
    GoUseTransparentColor();
end;

procedure TB2MWizard.OffsetNextButtonClick(Sender: TObject);
begin
  SetOffset(OffsetUpDown.Position);
  GoNoteType();
end;

procedure TB2MWizard.OffsetPrevButtonClick(Sender: TObject);
begin
  if UseTransparentColor then
    GoTransparentColor()
  else if ForceUseTransparentColor() then
    GoBitmapPath()
  else
    GoUseTransparentColor();
end;

procedure TB2MWizard.PrepareNoteTypeComboBox();
var
  n: TNoteType;
begin
  NoteTypeComboBox.Clear();
  for n := Low(TNoteType) to High(TNoteType) do
    NoteTypeComboBox.Items.Add(NoteTypeNames[n]);
end;

procedure TB2MWizard.SetResourceStrings();
begin
  PrevButton.Caption := resPrevButtonCaption;
  NextButton.Caption := resNextButtonCaption;
  CancelButton.Caption := resCancelButtonCaption;
  BitmapPathLabel.Caption := resBitmapPathLabelCaption;
  UseTransparentColorCheckBox.Caption := resUseTransparentColorCheckBoxCaption;
  TransparentColorLabel.Caption := resTransparentColorLabelCaption;
  OffsetLabel.Caption := resOffsetLabelCaption;
  NoteTypeLabel.Caption := resNoteTypeLabelCaption;
  NoteLengthLabel.Caption := resNoteLengthLabelCaption;
  TempoLabel.Caption := resTempoLabelCaption;
  MusPathLabel.Caption := resMusPathLabelCaption;
end;

procedure TB2MWizard.TransparentColorNextButtonClick(Sender: TObject);
begin
  TransparentColorIndex := TransparentColorListBox.ItemIndex;
  if CanSetOffset() then
    GoOffset()
  else
    GoNoteType();
end;

procedure TB2MWizard.TransparentColorPrevButtonClick(Sender: TObject);
begin
  if ForceUseTransparentColor() then
    GoBitmapPath()
  else
    GoUseTransparentColor();
end;

procedure TB2MWizard.UseTransparentColorNextButtonClick(Sender: TObject);
begin
  UseTransparentColor := UseTransparentColorCheckBox.Checked;
  if UseTransparentColor then
    GoTransparentColor()
  else if CanSetOffset() then
    GoOffset()
  else
    GoNoteType();
end;

procedure TB2MWizard.UseTransparentColorPrevButtonClick(Sender: TObject);
begin
   GoBitmapPath();
end;

procedure TB2MWizard.Execute();
begin
  GoBitmapPath();
end;

initialization
  {$I wizard.lrs}

end.

